<?php
   
namespace App\Http\Controllers\API;
   
use Illuminate\Http\Request;
use App\Http\Controllers\API\BaseController as BaseController;
use Validator;
use App\Http\Resources\Audiobook as AudiobookResource;
use App\Models\Audiobook;
use App\Models\User;
use App\Models\Wishlist;   
use App\Models\Userarchieve;   
   
class AudiobookController extends BaseController
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $audiobooks = Audiobook::with('audiobooktracks', 'author', 'languages')
                                ->whereHas('audiobooktracks')
                                ->where('is_active', 1);
                                
        if($request->get('author_id') != ''){
            $authorid = $request->get('author_id');
            $audiobooks = $audiobooks->where('author_id', $authorid);
        }
        
        $audiobooks = $audiobooks->get();
        
        if($audiobooks->count()){
            return $this->sendResponse(AudiobookResource::collection($audiobooks), 'Audiobook fetched successfully.');
        }else{
            return $this->sendError('No audiobooks found.', [], 400);
        }
    }
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // $input = $request->all();
   
        // $validator = Validator::make($input, [
        //     'name' => 'required|max:200'
        // ]);
   
        // if($validator->fails()){
        //     return $this->sendError('Validation Error.', $validator->errors());       
        // }
   
        // $author = new Author;
        // $author->name = $request->get('name');
        // $author->slug = $this->createUniqueSlug('Author', $request->get('name'));
        // $author->save();
   
        // return $this->sendResponse(new AuthorResource($author), 'Author created successfully.');
    } 
   
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $user = null;
        $user_id = request()->user_id;
        $listening_track = $userarchieve = null;
        $track_playing_time = 0;
        
        $audiobook = Audiobook::with('audiobooktracks', 'author', 'languages')
                                ->where('is_active', 1)->findOrFail($id);
        
        if(!is_null($user_id)){
            $user = User::find($user_id);
            if($user){
                $userarchieve = Userarchieve::whereUserId($user->id)->whereAudiobookId($audiobook->id)->first();
                
                if($userarchieve){
                    $listening_track = $userarchieve->audiobooktrack_id;
                }
            }
        }
        
        $total_duration = $total_size = 0;
        if($audiobook->audiobooktracks->count()){
           foreach($audiobook->audiobooktracks as $audiobooktrack) {
               $total_duration = $total_duration + $audiobooktrack->track_length; // in Sec
               $total_size = $total_size + $audiobooktrack->track_size; //in MB
               $audiobooktrack->track_playing_time = 0;
               if($userarchieve && $userarchieve->audiobooktrack_id == $audiobooktrack->id){
                    $audiobooktrack->track_playing_time = $userarchieve->track_playing_time;
                }
           }
        }
        
        if($total_duration > 0){
            $total_duration = $this->getFormattedTime($total_duration);
        }else{
            $total_duration = '00 Hr : 00 Min : 00 Sec';
        }
        
        if (is_null($audiobook)) {
            return $this->sendError('Audiobook not found.', 400);
        }
        
        
        if($user){
            $check = Wishlist::whereUserId($user->id)->whereAudiobookId($audiobook->id)->first();
            if($check){
                $audiobook->is_wishlisted = true;
            }else{
                $audiobook->is_wishlisted = false;
            }
        }else{
            $audiobook->is_wishlisted = false;
        }
        
        $audiobook->continue_listenting_track = $listening_track;
        $audiobook->total_duration = $total_duration;
        $audiobook->total_size = $this->getTotalSize($total_size);
        
        $audiobook = AudiobookResource::make($audiobook);
        
        
        return $this->sendResponse($audiobook, 'Audiobook retrieved successfully.');
        
    }
    
    protected function getFormattedTime($total_duration){
        $hour = $min = $sec = 0;
        if($total_duration >= 3600){
            $hour = (int)($total_duration/3600);
            $total_duration = fmod($total_duration,3600);
        }
        if($total_duration < 3600 && $total_duration >59){
            $min = (int)($total_duration/60);
            $sec = fmod($total_duration,60);
        }
        
        if(strlen($hour) == 1){
            $hour = '0' . $hour;
        }
        
        if(strlen($min) == 1){
            $min = '0' . $min;
        }
        
        if(strlen($sec) == 1){
            $sec = '0' . $sec;
        }
        
        return $hour . ' Hr : ' . $min . ' Min : ' . $sec . ' Sec';
    }
    
    protected function getTotalSize($total_size){
        if($total_size > 1023){
            $total_size = round($total_size/1024,2);
            return $total_size . ' GB';
        }else{
            return $total_size . ' MB';
        }
    }
}