<?php
   
namespace App\Http\Controllers\API;
   
use Illuminate\Http\Request;
use App\Http\Controllers\API\BaseController as BaseController;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Validator;
   
class RegisterController extends BaseController
{
    /**
     * Register api
     *
     * @return \Illuminate\Http\Response
     */
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|email',
            'password' => 'required',
            'c_password' => 'required|same:password',
        ]);
   
        if($validator->fails()){
            return $this->sendError('Validation Error.', $validator->errors());       
        }
   
        $input = $request->all();
        $input['password'] = bcrypt($input['password']);
        $user = User::create($input);
        $success['token'] =  $user->createToken('MyApp')->accessToken;
        $success['name'] =  $user->name;
   
        return $this->sendResponse($success, 'User register successfully.');
    }
   
    /**
     * Login api
     *
     * @return \Illuminate\Http\Response
     */
    public function login(Request $request)
    {
        if(Auth::attempt(['email' => $request->email, 'password' => $request->password])){ 
            $user = Auth::user(); 
            $success['token'] =  $user->createToken('MyApp')-> accessToken; 
            $success['name'] =  $user->firstname. ' '. $user->lastname;
   
            return $this->sendResponse($success, 'User login successfully.');
        } 
        else{ 
            return $this->sendError('Unauthorised.', ['error'=>'Unauthorised']);
        } 
    }
    
    // public function googleLogin()
    // {
    //     $firstname = request()->firstname;
    //     $lastname = request()->lastname ? request()->lastname : null;
    //     $email = request()->email;
    //     $gmail_token = request()->gmail_token;
    //     $error_data = [];
        
    //     if($firstname == ''){
    //         $error_data['firstname'] = "First name is required.";
    //     }
        
    //     if($email == ''){
    //         $error_data['email'] = "Email is required.";
    //     }
        
    //     if($gmail_token == ''){
    //         $error_data['gmail_token'] = "Gmail token is required.";
    //     }
        
    //     if(count($error_data) > 0){
    //         return $this->sendError('There is problem with your data.', $error_data);
    //     }
        
    //     $check_user = User::where('email', $email)->where('gmail_token', $gmail_token)->first();
        
    //     if($check_user){
    //         Auth::loginUsingId($check_user->id);
    //         $user = Auth::user();
    //         $check_user->firstname = $firstname;
    //         $check_user->lastname = $lastname;
    //         $check_user->update();
            
    //         $success['id'] =  $user->id;
    //         $success['token'] =  $user->createToken('MyApp')-> accessToken; 
    //         // $success['name'] =  $user->firstname. ' '. $user->lastname;
    //         $success['firstname'] =  $firstname;
    //         $success['lastname'] =  $lastname;
    //         $success['download_only_over_wifi'] = $user->download_only_over_wifi;
   
    //         return $this->sendResponse($success, 'User logged in successfully.');   
    //     }else{
    //         $check_user_without_token = User::where('email', $email)->first();
    //         if($check_user_without_token){
    //             $check_user_without_token->firstname = $firstname;
    //             $check_user_without_token->lastname = $lastname;
    //             $check_user_without_token->gmail_token = $gmail_token;
    //             $check_user_without_token->is_admin = 0;
    //             $check_user_without_token->update();
                
    //             Auth::loginUsingId($check_user_without_token->id);
    //             $success['id'] =  $check_user_without_token->id;
    //             $success['token'] =  $check_user_without_token->createToken('MyApp')-> accessToken; 
    //             // $success['name'] =  $check_user_without_token->firstname. ' '. $check_user_without_token->lastname;
    //             $success['firstname'] =  $firstname;
    //             $success['lastname'] =  $lastname;
    //             $success['download_only_over_wifi'] = $check_user_without_token->download_only_over_wifi;
                
    //             return $this->sendResponse($success, 'User logged in successfully.');
    //         }else{
    //             $user = User::create(['firstname' => $firstname, 'lastname' => $lastname, 'is_admin' => 0, 'email' => $email, 'gmail_token' => $gmail_token, 'download_only_over_wifi' => 'Yes']);
    //             Auth::login($user);
    //             $success['id'] =  $user->id;
    //             $success['token'] =  $user->createToken('MyApp')-> accessToken; 
    //             // $success['name'] =  $user->firstname. ' '. $user->lastname;
    //             $success['firstname'] =  $user->firstname;
    //             $success['lastname'] =  $user->lastname;
    //             $success['download_only_over_wifi'] = $user->download_only_over_wifi;
                
    //             return $this->sendResponse($success, 'User logged in successfully.'); 
    //         }
    //     }
        
    //     return $this->sendError('Unauthorised.', ['error'=>'Unauthorised']);
    // }
    
    public function socialLogin()
    {
        $firstname = request()->firstname;
        $lastname = request()->lastname ? request()->lastname : null;
        $email = request()->email;
        $gmail_token = request()->gmail_token;
        $facebook_token = request()->facebook_token;
        $error_data = [];
        
        if($firstname == ''){
            $error_data['firstname'] = "First name is required.";
        }
        
        if($email == ''){
            $error_data['email'] = "Email is required.";
        }
        
        if($gmail_token == '' && $facebook_token == ''){
            $error_data['error'] = "Choose any platform to login.";
        }
        
        if(count($error_data) > 0){
            return $this->sendError('There is problem with your data.', $error_data);
        }
        
        $check_user = User::where('email', $email);
        if($gmail_token != ''){
            $check_user = $check_user->where('gmail_token', $gmail_token);
        }else{
            $check_user = $check_user->where('fb_token', $facebook_token);
        }
        
        $check_user = $check_user->first();
        
        if($check_user){
            Auth::loginUsingId($check_user->id);
            $user = Auth::user();
            $check_user->firstname = $firstname;
            $check_user->lastname = $lastname;
            $check_user->update();
            
            $success['id'] =  $user->id;
            $success['token'] =  $user->createToken('MyApp')-> accessToken; 
            // $success['name'] =  $user->firstname. ' '. $user->lastname;
            $success['email'] =  $check_user->email ? $check_user->email : '';
            $success['firstname'] =  $firstname;
            $success['lastname'] =  $lastname;
            $success['download_only_over_wifi'] = $user->download_only_over_wifi;
   
            return $this->sendResponse($success, 'User logged in successfully.');   
        }else{
            $check_user_without_token = User::where('email', $email)->first();
            if($check_user_without_token){
                $check_user_without_token->firstname = $firstname;
                $check_user_without_token->lastname = $lastname;
                if($gmail_token != ''){
                    $check_user_without_token->gmail_token = $gmail_token;
                }else{
                    $check_user_without_token->fb_token = $facebook_token;
                }
                
                $check_user_without_token->is_admin = 0;
                $check_user_without_token->update();
                
                Auth::loginUsingId($check_user_without_token->id);
                $success['id'] =  $check_user_without_token->id;
                $success['token'] =  $check_user_without_token->createToken('MyApp')-> accessToken; 
                // $success['name'] =  $check_user_without_token->firstname. ' '. $check_user_without_token->lastname;
                $success['email'] =  $check_user_without_token->email ? $check_user_without_token->email : '';
                $success['firstname'] =  $firstname;
                $success['lastname'] =  $lastname;
                $success['download_only_over_wifi'] = $check_user_without_token->download_only_over_wifi;
                
                return $this->sendResponse($success, 'User logged in successfully.');
            }else{
                $data = [
                            'firstname' => $firstname, 
                            'lastname' => $lastname, 
                            'is_admin' => 0, 
                            'email' => $email, 
                            'gmail_token' => $gmail_token, 
                            // 'download_only_over_wifi' => 'Yes'
                        ];
                        
                if($gmail_token != ''){
                    $data['gmail_token'] = $gmail_token;
                }else{
                    $data['fb_token'] = $facebook_token;
                }
                
                $user = User::create($data);
                Auth::login($user);
                $success['id'] =  $user->id;
                $success['token'] =  $user->createToken('MyApp')-> accessToken; 
                // $success['name'] =  $user->firstname. ' '. $user->lastname;
                $success['email'] =  $user->email ? $user->email : '';
                $success['firstname'] =  $user->firstname;
                $success['lastname'] =  $user->lastname;
                $success['download_only_over_wifi'] = $user->download_only_over_wifi;
                
                return $this->sendResponse($success, 'User logged in successfully.'); 
            }
        }
        
        return $this->sendError('Unauthorised.', ['error'=>'Unauthorised']);
    }
    
    public function set_download_option(){
        $user_id = Auth::id();
        
        $user = User::find($user_id);
        if($user->download_only_over_wifi == 'Yes'){
            $user->download_only_over_wifi = 'No';
            $user->update();
            $success['download_only_over_wifi'] = false;
            
            return $this->sendResponse($success, 'Download can be possible over any network.'); ;
        }else{
            $user->download_only_over_wifi = 'Yes';
            $user->update();
            $success['download_only_over_wifi'] = true;
            
            return $this->sendResponse($success, 'Download can be possible only over wifi.'); ;
        }
    }
}