<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Audiobook;
use App\Models\Audiobooktrack;
use App\Models\Author;
use App\Models\Category;
use App\Models\Language;
use App\Models\Searchtag;
use File;
// use Maatwebsite\Excel\Facades\Excel;
// use App\Imports\AudiobookImport;

class AudiobookController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $categories = Category::orderBy('sortid')->get();
        $languages = Language::orderBy('sortid')->get();
        
        $category_val = $language_val = $author_val = '';
        
        $audiobooks = Audiobook::with('author', 'categories', 'languages');
        
        if($request->get('category') != ''){
            $category_val = $request->category;
            /*$audiobooks =$audiobooks->with(['categories' => function($q) use ($category_val){
                                        $q->where('id', $category_val);
                                    }]);*/
            $audiobooks =$audiobooks->whereHas('categories', function($query) use($category_val){
                $query->where('id', $category_val);
            });
            // $audiobooks = $audiobooks->with('getfilteredCategories');
        }
        
        if($request->get('language') != ''){
            $language_val = $request->language;
            /*$audiobooks =$audiobooks->with(['languages' => function($q) use ($language_val){
                $q->where('short_code', $language_val);
            }]);*/
            $audiobooks =$audiobooks->whereHas('languages', function($query) use($language_val){
                $query->where('short_code', $language_val);
            });
        }
        
        if($request->has('author') && $request->author != ''){
            $author_val = $request->author;
            $get_author_id = Author::where('name','like','%'.$author_val.'%')->pluck('id');
            $audiobooks = $audiobooks->where(function($q) use ($get_author_id,$author_val){
                                            $q->whereIn('author_id',$get_author_id)->orWhere('name','like','%'.$author_val.'%');
                                        });
        }
        
        /*if($category_val == '' && $language_val == '' && $author_val == ''){
            $audiobooks = $audiobooks->with(['author', 'categories', 'languages']);
        }*/
        
        $audiobooks = $audiobooks->orderBy('id', 'DESC');
        
        $audiobooks = $audiobooks->paginate(50);
        
        return view('admin.audiobook.index',compact('audiobooks','languages','categories','category_val','language_val','author_val'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::getActiveCategories()->orderBy('sortid')->get();
        $authors = Author::orderBy('name')->get();
        $languages = Language::orderBy('sortid')->get();
        
        return view('admin.audiobook.create',compact('categories','authors','languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $current_year = date('Y');
        $validateData = $request->validate([
            'audio_name' => 'required|max:255',
            'category' => 'required',
            'language' => 'required',
            'year' => 'required|numeric|max:'.$current_year,
            'sku' => 'required|unique:audiobooks,sku,NULL,id,deleted_at,NULL',
            'description' => 'required',
            'file' => 'required|mimes:jpeg,jpg,png',
            ]);
        
        $file = $request->file('file');
        $destinationPath = 'audiobooks';
        if(!empty($file)){
            $filename_cv=uniqid().'.'.$file->getClientOriginalExtension();
            $file->move(
                base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.$destinationPath, $filename_cv
            );
        }
        // $fileextension = $file->getClientOriginalExtension();
        // $filename_cvl='audiobook'.uniqid().'.'.$fileextension;
        // $file->move('public/images',$filename_cvl);
        
        $audiobook = Audiobook::create([
            'name' => $request->audio_name,
            'author_id' => $request->author,
            'storagedrive_id' => 1,
            'sku' => strtoupper($request->sku),
            'publication_year' => $request->year,
            'thumb_photo' => $filename_cv,
            'description' => $request->description
            ]);
        $audiobook->categories()->sync($request->category);
        $audiobook->languages()->sync($request->language);
        if($request->has('search_tags') && $request->search_tags != ''){
            $search_tags = explode(',',$request->search_tags);
            $search_id = [];
            foreach($search_tags as $search_tag){
                $res = Searchtag::where('searchtag', $search_tag)->first();
                if($res){
                    $search_id[] = $res->id;
                }else{
                    $searchtag = Searchtag::create(['searchtag' => $search_tag]);
                    $search_id[] = $searchtag->id;
                }
            }
            $audiobook->searchtags()->sync($search_id);
        }
        $redirect = route('admin.audiobook.show',[\Crypt::encrypt($audiobook->id)]);
        if($audiobook){
            return response()->json(['success' => true, 'message' => 'Audiobook added successfully.', 'redirectto' => $redirect]);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }  
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try{
            $id = \Crypt::decrypt($id);
            $audiobook = Audiobook::with(['audiobooktracks' => function($q){
                $q->orderBy('sortid');
            }])->find($id);
            
            $categories = Category::orderBy('sortid')->get();
            $languages = Language::orderBy('sortid')->get();
            $authors = Author::orderBy('name')->get();
            
            return view('admin.audiobook.show', compact('audiobook','categories','languages','authors'));
        }catch (\Exception $e) {
            abort(404);
        }
        
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Audiobook $audiobook)
    {   
        if($audiobook){
            $previous_img = $audiobook->thumb_photo;
            $current_year = date('Y');
            $validateData = $request->validate([
                'audio_name' => 'required|max:255',
                'category' => 'required',
                'language' => 'required',
                'year' => 'required|numeric|max:'.$current_year,
                'sku' => 'required|unique:audiobooks,sku,'.$audiobook->id.',id,deleted_at,NULL',
                'description' => 'required',
                'file' => 'nullable|mimes:jpeg,jpg,png',
                ]);
            if($request->has('author') && $request->author != ''){
                $request->validate([
                    'author' => 'nullable|max:255'
                ]);
                
                if(is_numeric($request->author)){
                    $chk = Author::find($request->author);
                    if($chk){
                        $audiobook->author_id = $request->author;
                    }
                }else{
                    $slug = $this->createUniqueSlug($request->author, 0);
        
                    $author = Author::create([
                        'name' => $request->get('author'),
                        'slug' => $slug
                        ]);
                    $audiobook->author_id = $author->id;
                }
                
            }
            $audiobook->name =  $request->audio_name;
            $audiobook->sku =  $request->sku;
            $audiobook->publication_year =  $request->year;
            $audiobook->description =  $request->description;
            if($request->has('file') && !empty($request->file('file'))){
                $file = $request->file('file');
                $destinationPath = 'audiobooks';
                $filename_cv=uniqid().'.'.$file->getClientOriginalExtension();
                $file->move(
                    base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.$destinationPath, $filename_cv
                );
                // $fileextension = $file->getClientOriginalExtension();
                // $filename_cvl='audiobook'.uniqid().'.'.$fileextension;
                // $file->move('public/images',$filename_cvl);  
                
                $audiobook->thumb_photo = $filename_cv;
                
                if (File::exists(base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'audiobooks'.DIRECTORY_SEPARATOR.$previous_img));
                {
                    File::delete(base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'audiobooks'.DIRECTORY_SEPARATOR.$previous_img);
                }
            
                
            }
            $audiobook->update();
            
            $audiobook->categories()->sync($request->category);
            $audiobook->languages()->sync($request->language);
            if($request->has('search_tags') && $request->search_tags != ''){
                $search_tags = explode(',',$request->search_tags);
                $search_id = [];
                foreach($search_tags as $search_tag){
                    $res = Searchtag::where('searchtag', $search_tag)->first();
                    if($res){
                        $search_id[] = $res->id;
                    }else{
                        $searchtag = Searchtag::create(['searchtag' => $search_tag]);
                        $search_id[] = $searchtag->id;
                    }
                }
                $audiobook->searchtags()->sync($search_id);
            }
            
            return response()->json(['success' => true, 'message' => 'Audiobook updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Audiobook $audiobook)
    {
        if($audiobook){
            $audiobook->delete();
            return response()->json(['success' => true, 'message' => 'Audiobook deleted successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
    }
    
    public function changeStatus(Audiobook $audiobook){
        if($audiobook){
            $audiobook->is_active = $audiobook->is_active == 1 ? 0 : 1;
            $audiobook->update();
            
            return response()->json(['success' => true, 'message' => 'Audiobook status has been changed.']);
        }
        
        return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
    }
    
    public function createUniqueSlug($title, $id){
        $slug = preg_replace("/-$/","",preg_replace('/[^a-z0-9]+/i', "-", strtolower($title)));
        $modelname = Author::where('slug', 'LIKE', $slug.'%')->where('id', '!=', $id)
                                ->get();
        $numHits = $modelname->count();
        
        return ($numHits > 0) ? ($slug . '-' . $numHits) : $slug;
    }
    
    public function import(Request $request){
        $book_lists = $request->book_list[0];
        $track_lists = $request->track_list[0];
        $is_inserted = false;
        
        foreach($book_lists as $key=>$book_list){
            if(isset($book_list['sku']) && isset($book_list['name']) && isset($book_list['categories']) && isset($book_list['language']) && isset($book_list['publication_year']) && isset($book_list['thumb_photo']) && isset($book_list['description']) && $book_list['sku'] != '' && $book_list['name'] != '' && $book_list['categories'] != '' && $book_list['language'] != '' && $book_list['publication_year'] !=''  && $book_list['thumb_photo'] !=''  && $book_list['description'] !=''){
                $chk_sku = Audiobook::where('sku', $book_list['sku'])->first();
                if(!$chk_sku){
                    // return response()->json(['success' => false, 'message' => $book_list['sku'] .' is already exist in row '. $key+2 . ' in Books sheet.']);
                    $chk_img = Audiobook::where('thumb_photo',$book_list['thumb_photo'])->first();
                    if($chk_img){
                        return response()->json(['success' => false, 'message' => $book_list['thumb_photo'] .' is already exist in row '. ($key+2) . ' in the Books sheet.']);
                    }
                    
                    $categories = explode(' | ', $book_list['categories']);
                    $cat_id= [];
                    foreach($categories as $category){
                        $is_exist_cat = Category::where('name',$category)->first();
                        if(!$is_exist_cat){
                            return response()->json(['success' => false, 'message' => $category .' does not exist in row '. ($key+2) . ' in the Books sheet.']);
                        }else{
                            $cat_id[] = $is_exist_cat->id;
                        }
                    }
                    
                    /*$languages = explode(' | ', $book_list['language']);
                    $lang_id = [];
                    foreach($languages as $language){
                        $is_exist_lang = Language::where('name', $book_list['language'])->first();
                        if(!$is_exist_lang){
                            return response()->json(['success' => false, 'message' => $language .' does not exist in row '. ($key+2) . ' in the Books sheet.']);
                        }else{
                            $lang_id[] = $is_exist_lang->id;
                        }
                    }*/
                    
                    $chk_lang = Language::where('name', $book_list['language'])->first();
                    if(!$chk_lang){
                        return response()->json(['success' => false, 'message' => $book_list['language'] .' does not exist in row '. ($key+2) . ' in the Books sheet.']);
                    }else{
                        $lang_id = $chk_lang->id;
                    }
                    
                    if(is_numeric($book_list['publication_year']) && $book_list['publication_year'] <= date('Y')){
                        $publication_year = $book_list['publication_year'];
                    }else{
                        return response()->json(['success' => false, 'message' => $book_list['publication_year'] .' is invalid in row '. ($key+2) . ' in the Books sheet.']);
                    }
                    
                    if(isset($book_list['author_name']) && $book_list['author_name'] != ''){
                        $chk = Author::where('name', $book_list['author_name'])->first();
                        if($chk){
                            $author_id = $chk->id;
                        }else{
                            $slug = $this->createUniqueSlug($book_list['author_name'], 0);
                
                            $author = Author::create([
                                'name' => $book_list['author_name'],
                                'slug' => $slug
                                ]);
                            $author_id = $author->id;
                        }
                    }else{
                        $author_id = NULL;
                    }
                    
                    // Book list validation complete
                    
                    $is_track_present = $this->checkSkuInTrackList($book_list['sku'], $track_lists);
                    if($is_track_present['status']){
                        $audiobook = Audiobook::create([
                                'name' => $book_list['name'],
                                'author_id' => $author_id,
                                'storagedrive_id' => 1,
                                'sku' => strtoupper($book_list['sku']),
                                'publication_year' => $book_list['publication_year'],
                                'thumb_photo' => $book_list['thumb_photo'],
                                'description' => $book_list['description']
                            ]);
                        
                        $audiobook->categories()->sync($cat_id);
                        $audiobook->languages()->sync($lang_id);
                        if(isset($book_list['search_tags']) && $book_list['search_tags'] != ''){
                            $search_tags = explode(' | ',$book_list['search_tags']);
                            $search_id = [];
                            foreach($search_tags as $search_tag){
                                $res = Searchtag::where('searchtag', $search_tag)->first();
                                if($res){
                                    $search_id[] = $res->id;
                                }else{
                                    $searchtag = Searchtag::create(['searchtag' => $search_tag]);
                                    $search_id[] = $searchtag->id;
                                }
                            }
                            $audiobook->searchtags()->sync($search_id);
                        }
                            
                        $audiobook_track = $this->insertTrackData($audiobook, $track_lists);
                        if($audiobook_track['status']){
                            $is_inserted = true;
                        }else{
                            if($audiobook_track['required_field']){
                                return response()->json(['success' => false, 'message' => 'Please fill the required in the Tracks sheet.']);
                            }
                            if(strlen($audiobook_track['message']) != ''){
                                return response()->json(['success' => false, 'message' => $audiobook_track['message']]); 
                            }
                            return response()->json(['success' => false, 'message' => 'Something went wrong.']);
                        }
                    }else{
                        if($is_track_present['required_field']){
                            return response()->json(['success' => false, 'message' => 'Please fill the required in the Tracks sheet.']);
                        }
                        if(strlen($is_track_present['message']) != ''){
                            return response()->json(['success' => false, 'message' => $is_track_present['message']]); 
                        }
                        return response()->json(['success' => false, 'message' => 'Please enter track for '.$book_list['sku'] . ' with valid data in the Tracks sheet.']);
                    }
                    
                }else{
                    $will_update = $this->addTracks($chk_sku, $track_lists);
                    if($will_update['status']){
                        $is_inserted =true;
                    }else{
                        if($will_update['required_field']){
                            return response()->json(['success' => false, 'message' => 'Please fill the required in the Tracks sheet.']);
                        }
                        if(strlen($will_update['message']) != ''){
                            return response()->json(['success' => false, 'message' => $will_update['message']]); 
                        }
                        // return response()->json(['success' => false, 'message' => 'Something went wrong.']);
                    }
                }
                
            }else{
                return response()->json(['success' => false, 'message' => 'Please fill the data in row '. ($key+2) . ' in Books sheet.']);
            }     
            
        }
        
        if($is_inserted){
            return response()->json(['success' => true, 'message' => 'You have successfully imported.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Data is already present.']);
        }
    }
    
    public function checkSkuInTrackList($sku, $track_lists){
        $result['status'] = $result['required_field'] = false;
        $result['message'] = '';
        foreach($track_lists as $key=>$list){
            if(isset($list['sku']) && isset($list['track_number']) && isset($list['track_title']) && isset($list['file_url'])){
                if($list['sku'] == $sku){
                    if(is_numeric($list['track_number']) && $list['track_number'] != '' &&  $list['track_title'] != '' && $list['file_url'] != '' && filter_var($list['file_url'], FILTER_VALIDATE_URL)){
                        if(isset($list['track_length']) && !is_numeric($list['track_length'])){
                            $result['message'] = 'Invalid track length in row number '.($key+2).'in Tracks sheet';
                            $result['status'] = false; 
                            return $result;
                        }
                        if(isset($list['track_size']) && !is_numeric($list['track_size'])){
                            $result['status'] = false; 
                            $result['message'] = 'Invalid track size in row number '.($key+2).'in Tracks sheet';
                            return $result;
                        }
                        $result['status'] = true;    
                        return $result;
                    }else{
                        $result['status'] = false; 
                        $result['message'] = 'Invalid track number in row number '.($key+2).'in Tracks sheet';
                        return $result;
                    }
                }
            }else{
                $result['required_field'] = true;
                return $result;
            }
        }
        
        return $result;
    }
    
    public function insertTrackData($audiobook, $track_lists){
        $result['status'] = $result['required_field'] = false;
        $result['message'] = '';
        foreach($track_lists as $key=>$list){
            if(isset($list['sku']) && isset($list['track_number']) && isset($list['track_title']) && isset($list['file_url'])){
                if($list['sku'] == $audiobook->sku){
                    if(is_numeric($list['track_number']) && $list['track_number'] != '' &&  $list['track_title'] != '' && $list['file_url'] != '' && filter_var($list['file_url'], FILTER_VALIDATE_URL)){
                        if(isset($list['track_length']) && !is_numeric($list['track_length'])){
                            $result['message'] = 'Invalid track length in row number '.($key+2).'in Tracks sheet';
                            $result['status'] = false; 
                            return $result;
                        }
                        if(isset($list['track_size']) && !is_numeric($list['track_size'])){
                            $result['status'] = false; 
                            $result['message'] = 'Invalid track size in row number '.($key+2).'in Tracks sheet';
                            return $result;
                        }
                        $track = Audiobooktrack::create([
                                    'audiobook_id' => $audiobook->id,
                                    'track_title' => $list['track_title'],
                                    'file_url' => $list['file_url'],
                                    'track_length' => isset($list['track_length']) ? $list['track_length'] : null,
                                    'track_size' => isset($list['track_size']) ? $list['track_size'] : null,
                                    'sortid' => $list['track_number']
                                ]);
                        $result['status'] = true;    
                        // return $result;
                    }else{
                        $result['status'] = false; 
                        $result['message'] = 'Invalid track number in row number '.($key+2).'in Tracks sheet';
                        return $result;
                    }
                }
            }else{
                $result['required_field'] = true;
                return $result;
            }
            /*if($list['sku'] == $audiobook->sku){
                if($list['track_number'] != '' &&  $list['track_title'] != '' && $list['file_url'] != '' && filter_var($list['file_url'], FILTER_VALIDATE_URL)){
                    $track = Audiobooktrack::create([
                                'audiobook_id' => $audiobook->id,
                                'track_title' => $list['track_title'],
                                'file_url' => $list['file_url'],
                                'track_length' => $list['track_length'],
                                'track_size' => $list['track_size'],
                                'sortid' => $list['track_number']
                            ]);
                }
            }*/
        }
        return $result;
    }
    
    public function addTracks($audiobook, $track_lists){
        $result['status'] = $result['required_field'] = false;
        $result['message'] = '';
        foreach($track_lists as $key=>$list){
            if(isset($list['sku']) && isset($list['track_number']) && isset($list['track_title']) && isset($list['file_url'])){
                if($list['sku'] == $audiobook->sku){
                    if(is_numeric($list['track_number']) && $list['track_number'] != '' &&  $list['track_title'] != '' && $list['file_url'] != '' && filter_var($list['file_url'], FILTER_VALIDATE_URL)){
                        if(isset($list['track_length']) && !is_numeric($list['track_length'])){
                            $result['message'] = 'Invalid track length in row number '.($key+2).'in Tracks sheet';
                            $result['status'] = false; 
                            return $result;
                        }
                        if(isset($list['track_size']) && !is_numeric($list['track_size'])){
                            $result['status'] = false; 
                            $result['message'] = 'Invalid track size in row number '.($key+2).'in Tracks sheet';
                            return $result;
                        }
                        $check_is_exist = Audiobooktrack::where('file_url', $list['file_url'])->where('audiobook_id', $audiobook->id)->first();
                        if(!$check_is_exist){
                            $track = Audiobooktrack::create([
                                    'audiobook_id' => $audiobook->id,
                                    'track_title' => $list['track_title'],
                                    'file_url' => $list['file_url'],
                                    'track_length' => isset($list['track_length']) ? $list['track_length'] : null,
                                    'track_size' => isset($list['track_size']) ? $list['track_size'] : null,
                                    'sortid' => $list['track_number']
                                ]);
                            $result['status'] = true;  
                        }
                          
                        // return $result;
                    }else{
                        $result['status'] = false; 
                        $result['message'] = 'Invalid track number in row number '.($key+2).'in Tracks sheet';
                        return $result;
                    }
                }
            }else{
                $result['required_field'] = true;
                return $result;
            }
            /*if($list['sku'] == $audiobook->sku){
                if($list['track_number'] != '' &&  $list['track_title'] != '' && $list['file_url'] != '' && filter_var($list['file_url'], FILTER_VALIDATE_URL)){
                    $check_is_exist = Audiobooktrack::where('file_url', $list['file_url'])->where('audiobook_id', $audiobook->id)->first();
                    if(!$check_is_exist){
                        $track = Audiobooktrack::create([
                                'audiobook_id' => $audiobook->id,
                                'track_title' => $list['track_title'],
                                'file_url' => $list['file_url'],
                                'track_length' => $list['track_length'],
                                'track_size' => $list['track_size'],
                                'sortid' => $list['track_number']
                            ]);
                            $is_updated = true;
                    }
                    
                }
            }*/
        }
        return $result;
    }
    
    public function importimages(Request $request){
        $request->validate([
            'images' => 'required'
            ]);
        $upload_status = false;
        $image_name = [];
        if($request->hasFile('images'))
        {
            $files = $request->file('images');
            foreach($files as $file)
            {
                $destinationPath = 'audiobooks';
                if(!empty($file)){
                    $originalname = $file->getClientOriginalName();
                    $fileextension = $file->getClientOriginalExtension();
                    $filename_cv = $originalname;
                    $chk_img_exist = Audiobook::where('thumb_photo', $filename_cv)->first();
                    
                    if($chk_img_exist){
                        $file->move(
                            base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.$destinationPath, $filename_cv
                        );
                        $upload_status = true;
                    }else{
                        $image_name[] = $filename_cv;
                    }  
                }
            }
        }
        
        if($upload_status){
            if(count($image_name) == 0){
                return response()->json(['success' => true, 'message' => 'Images uploaded Successfully']);    
            }else{
                return response()->json(['success' => true, 'message' => 'Some images do not exist in the system. <br>'. implode(', ', $image_name)]);    
            }
            
        }else{
            if(count($image_name) == 0){
                return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);   
            }else{
                return response()->json(['success' => false, 'message' => 'Images with the provided names are not present in the system.']);    
            }
            
        }
        
    }
}
