<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Author;

class AuthorController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $authors = Author::orderBy('name')->get();
        
        return view('admin.author.index', compact('authors'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.author.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validateData = $request->validate([
            'name' => 'required|max:255'
            ]);
            
        $slug = $this->createUniqueSlug($request->name, 0);
        
        $author = Author::create([
            'name' => $request->get('name'),
            'slug' => $slug
            ]);
        if($author){
            return response()->json(['success' => true, 'message' => 'Author added successfully.', 'author' => $author]);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }  
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validateData = $request->validate([
            'name' => 'required|max:255'
            ]);
            
        $slug = $this->createUniqueSlug($request->name, $id);
        
        $author = Author::find($id);
        if($author){
            $author->name = $request->get('name');
            $author->slug = $slug;
            
            $author->update();
            
            return response()->json(['success' => true, 'message' => 'Author updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $author = Author::find($id);
        if($author){
            $author->delete();
            return response()->json(['success' => true, 'message' => 'Author deleted successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
    }
    
    public function createUniqueSlug($title, $id){
        $slug = preg_replace("/-$/","",preg_replace('/[^a-z0-9]+/i', "-", strtolower($title)));
        $modelname = Author::where('slug', 'LIKE', $slug.'%')->where('id', '!=', $id)
                                ->get();
        $numHits = $modelname->count();
        
        return ($numHits > 0) ? ($slug . '-' . $numHits) : $slug;
    }
}
