<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use File;

class CategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $categories = Category::with('audiobooks')->orderBy('sortid')->get()->toArray();
        
        return view('admin.category.index', compact('categories'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.category.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validateData = $request->validate([
            'name' => 'required|max:255||unique:categories,name,NULL,id,deleted_at,NULL',
            'file' => 'required|mimes:jpeg,jpg,png'
            ]);
        $file = $request->file('file');
        $destinationPath = 'categories';
        if(!empty($file)){
            $filename_cv=uniqid().'.'.$file->getClientOriginalExtension();
            $file->move(
                base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.$destinationPath, $filename_cv
            );
        }
        // $fileextension = $file->getClientOriginalExtension();
        // $filename_cvl='category'.uniqid().'.'.$fileextension;
        // $file->move('public/images',$filename_cvl);  
        
        $sort_id = Category::max('sortid') + 1;
        
        $category = Category::create([
            'name' => $request->get('name'),
            'image' => $filename_cv,
            'sortid' => $sort_id
            ]);
        if($category){
            return response()->json(['success' => true, 'message' => 'Category added successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }    
                    
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validateData = $request->validate([
            'name' => 'required|max:255|unique:categories,name,'.$id.',id,deleted_at,NULL',
            'file' => 'nullable|mimes:jpeg,jpg,png'
            ]);
        
        $category = Category::find($id);
        
        if($category){
            $prev_image_link = $category->image;
            $category->name = $request->get('name');
            
            if($request->has('file') && !empty($request->file('file')))
            {
                $file = $request->file('file');
                $destinationPath = 'categories';
                $filename_cv=uniqid().'.'.$file->getClientOriginalExtension();
                $file->move(
                    base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.$destinationPath, $filename_cv
                );
                
                $category->image = $filename_cv;
                if (File::exists(base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'categories'.DIRECTORY_SEPARATOR.$prev_image_link));
                {
                    File::delete(base_path().DIRECTORY_SEPARATOR.'public'.DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'categories'.DIRECTORY_SEPARATOR.$prev_image_link);
                }
            }
            $category->update();
            
            return response()->json(['success' => true, 'message' => 'Category updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        } 
    }
    
    public function setFeaturedStatus(Category $category)
    {
        if($category){
            if($category->is_featured == 'No'){
                $check_featured_category_count = Category::getActiveCategories()->where('is_featured', 'Yes')->count();
                if($check_featured_category_count < 5){
                    $category->is_featured = 'Yes';
                    $category->update();
                    
                    return response()->json(['success' => true, 'message' => 'Category is now marked as featured.']);
                }
                return response()->json(['success' => false, 'message' => 'You can only mark five categories as featured.']);
            }else{
                $category->is_featured = 'No';
                $category->update();
                
                return response()->json(['success' => true, 'message' => 'Category is now removed from featured.']);
            }
        }
        return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $category = Category::find($id);
        if($category){
            if($category->audiobooks->count()){
                return response()->json(['success' => false, 'message' => 'You can not delete this category as it is associated with audiobook.']);
            }else{
                $category->delete();
                return response()->json(['success' => true, 'message' => 'Category deleted successfully.']);
            }
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
    }
    
    public function changeStatus(Request $request, Category $category){
        if($category){
            
            $category->is_active = $category->is_active == 1 ? 0 : 1;
            $category->is_featured = 'No';
            $category->update();
            
            return response()->json(['success' => true, 'message' => 'Category status updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
        
    }
    
    public function arrangeCategoryInOrder(Request $request){
        foreach($request->positions as $position){
            Category::where('id', $position[0])->update(['sortid' => $position[1]]);
        }
        return response()->json(true);
    }
}
