<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Language;

class LanguageController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $languages = Language::orderBy('sortid')->get();
        
        return view('admin.language.index', compact('languages'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.language.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validateData = $request->validate([
            'name' => 'required|max:255|unique:languages,name,NULL,id,deleted_at,NULL',
            'short_code' => 'required|max:255|unique:languages,short_code,NULL,id,deleted_at,NULL'
            ]);
        
        $sort_id = Language::max('sortid') + 1;
        
        $language = Language::create([
            'name' => ucfirst($request->get('name')),
            'short_code' => strtoupper($request->get('short_code')),
            'sortid' => $sort_id
            ]);
        if($language){
            return response()->json(['success' => true, 'message' => 'Language added successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }  
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Language $language)
    {
        $validateData = $request->validate([
            'name' => 'required|max:255|unique:languages,name,'.$language->id.',id,deleted_at,NULL',
            'short_code' => 'required|max:255|unique:languages,short_code,'.$language->id.',id,deleted_at,NULL'
            ]);
        if($language){
            $language->name = ucfirst($request->get('name'));
            $language->short_code = strtoupper($request->get('short_code'));
            
            $language->update();
            return response()->json(['success' => true, 'message' => 'Language updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $language = Language::find($id);
        if($language){
            if($language->audiobooks->count()){
                return response()->json(['success' => false, 'message' => 'You can not delete this language as it is associated with audiobook.']);
            }else{
                $language->delete();
                return response()->json(['success' => true, 'message' => 'Language deleted successfully.']);
            }
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }
    }
    
    public function arrangeCategoryInOrder(Request $request){
        foreach($request->positions as $position){
            Language::where('id', $position[0])->update(['sortid' => $position[1]]);
        }
        return response()->json(true);
    }
}
