<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\Popularbook;

class PopularbookController extends Controller
{
    public function index(Request $request)
    {
        $popularbooks = Popularbook::whereHas('audiobook', function($q){
            $q->where('is_active',1);
        })->orderBy('sortid')->get();
        return view('admin.popularbook.index',compact('popularbooks'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::getActiveCategories()->orderBy('sortid')->get();
        
        return view('admin.popularbook.create',compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validateData = $request->validate([
            'books' => 'required'
            ]);
        
        $sortid = Popularbook::max('sortid') + 1;
        foreach($request->books as $book){
            $popularbook = Popularbook::create([
                'audiobook_id' => $book,
                'sortid' => $sortid
                ]);
            $sortid++;
        }
            
        if($popularbook){
            return response()->json(['success' => true, 'message' => 'Popularbook added successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
        }  
    }
    
    public function destroy(Popularbook $popularbook){
        if($popularbook){
            $popularbook->delete();
            return response()->json(['success' => true, 'message' => 'Popularbook deleted successfully.']);
        }
        return response()->json(['success' => false, 'message' => 'Something went wrong. Please try again.']);
    }
    
    public function fetchBooks(Request $request){
        $books = Category::with('audiobooks')->where('id', $request->id)->get()->toArray();
        $options = '<option value="">Select Books</option>';
        
        if(count($books[0]['audiobooks'])){
            foreach($books[0]['audiobooks'] as $book){
                if($book['is_active'] == 1){
                    $is_present = Popularbook::where('audiobook_id',$book['id'])->first();
                    if(!$is_present){
                        $options .= '<option value="'.$book['id'].'">'.$book['name'].'</option>';    
                    }
                }
            }
        }
        
        return response()->json(['success' => true, 'data' => $options]);
        
    }
}
