<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\HomeController;
use App\Http\Controllers\Auth\LoginController;
use App\Http\Controllers\Admin\CategoryController;
use App\Http\Controllers\Admin\LanguageController;
use App\Http\Controllers\Admin\AuthorController;
use App\Http\Controllers\Admin\AudiobookController;
use App\Http\Controllers\Admin\AudiobooktrackController;
use App\Http\Controllers\Admin\UserController;
use App\Http\Controllers\Admin\PopularbookController;
/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

/*Route::get('/', function () {
     return view('welcome');
});*/

Route::get('/scan', [App\Http\Controllers\MaintainanceController::class, 'getDirs']);

Auth::routes();

// Route::get('/home', [HomeController::class, 'index'])->name('home');
Route::get('/admin-login', [HomeController::class, 'login'])->name('admin.login');
Route::post('/admin-login', [LoginController::class, 'loginProcess'])->name('admin.login.process');

Route::group(["middleware" => 'is_admin', "prefix" => 'admin'], function(){
    Route::get('logout', [LoginController::class, 'logout'])->name('logout');
    // Route::get('home', [HomeController::class, 'adminHome'])->name('admin.home');    
    Route::resource('categories', CategoryController::class)->names([
        'index' => 'admin.categories.index', 
        'create' => 'admin.categories.create', 
        'store' => 'admin.categories.store', 
        'show' => 'admin.categories.show', 
        'edit' => 'admin.categories.edit', 
        // 'update' => 'admin.categories.update', 
        'destroy' => 'admin.categories.destroy'
        ]);
    
    Route::post('/categories/update/{category}', [CategoryController::class, 'update'])->name('admin.categories.update');
    Route::post('/categories/update_status/{category}', [CategoryController::class, 'changeStatus'])->name('admin.categories.update-status');
    Route::post('/categories/is_featured/{category}', [CategoryController::class, 'setFeaturedStatus'])->name('admin.categories.set-featured-status');
    Route::post('/categories/sorting', [CategoryController::class, 'arrangeCategoryInOrder'])->name('admin.categories.sorting');
        
    Route::resource('languages', LanguageController::class)->names([
        'index' => 'admin.languages.index', 
        'create' => 'admin.languages.create', 
        'store' => 'admin.languages.store', 
        'show' => 'admin.languages.show', 
        'edit' => 'admin.languages.edit', 
        // 'update' => 'admin.languages.update', 
        'destroy' => 'admin.languages.destroy'
    ]);
    Route::post('/languages/update/{language}', [LanguageController::class, 'update'])->name('admin.languages.update');
    Route::post('/languages/sorting', [LanguageController::class, 'arrangeCategoryInOrder'])->name('admin.languages.sorting');
    
    Route::get('/author', [AuthorController::class, 'index'])->name('admin.author.index');
    Route::get('/author/create', [AuthorController::class, 'create'])->name('admin.author.create');
    Route::post('/author/store', [AuthorController::class, 'store'])->name('admin.author.store');
    Route::post('/author/update/{id}', [AuthorController::class, 'update'])->name('admin.author.update');
    Route::delete('/author/destroy/{id}', [AuthorController::class, 'destroy'])->name('admin.author.destroy');
    
    Route::get('/audiobook', [AudiobookController::class, 'index'])->name('admin.audiobook.index');
    Route::get('/audiobook/create', [AudiobookController::class, 'create'])->name('admin.audiobook.create');
    Route::post('/audiobook/store', [AudiobookController::class, 'store'])->name('admin.audiobook.store');
    Route::get('/audiobook/show/{id}', [AudiobookController::class, 'show'])->name('admin.audiobook.show');
    Route::post('/audiobook/update/{audiobook}', [AudiobookController::class, 'update'])->name('admin.audiobook.update');
    Route::delete('/audiobook/destroy/{audiobook}', [AudiobookController::class, 'destroy'])->name('admin.audiobook.destroy');
    Route::post('/audiobook/update_status/{audiobook}', [AudiobookController::class, 'changeStatus'])->name('admin.audiobook.update-status');
    Route::post('/audiobook/import', [AudiobookController::class, 'import'])->name('admin.audiobook.import');
    Route::view('/audiobook/import-images', 'admin/audiobook/importimages')->name('import-images');
    Route::post('/audiobook/import-images', [AudiobookController::class, 'importimages'])->name('admin.audiobook.import_bulk_images');
    
    Route::post('/audiobooktrack/store', [AudiobooktrackController::class, 'store'])->name('admin.audiobooktrack.store');
    Route::post('/audiobooktrack/update/{id}', [AudiobooktrackController::class, 'update'])->name('admin.audiobooktrack.update');
    Route::delete('/audiobooktrack/destroy/{id}', [AudiobooktrackController::class, 'destroy'])->name('admin.audiobooktrack.destroy');
    Route::post('/audiobooktrack/sorting', [AudiobooktrackController::class, 'arrangeCategoryInOrder'])->name('admin.audiobooktrack.sorting');
    
    Route::get('/popularbook', [PopularbookController::class, 'index'])->name('admin.popularbook.index');
    Route::get('/popularbook/create', [PopularbookController::class, 'create'])->name('admin.popularbook.create');
    Route::post('/popularbook/store', [PopularbookController::class, 'store'])->name('admin.popularbook.store');
    Route::post('/popularbook/update/{id}', [PopularbookController::class, 'update'])->name('admin.popularbook.update');
    Route::delete('/popularbook/destroy/{popularbook}', [PopularbookController::class, 'destroy'])->name('admin.popularbook.destroy');
    Route::post('/popularbook/fetchbooks', [PopularbookController::class, 'fetchBooks'])->name('admin.fetchbooks-byCategory');
    
    Route::get('/users',[UserController::class, 'index'])->name('admin.users');
    Route::get('/dashboard',[UserController::class, 'dashboard'])->name('admin.dashboard');
    
});

